@@ -6,7 +6,7 @@
 #' argument. So we need to check for the error file to decide
 #' if an error happened.
 #'
-#' @param out List of the output object from [run()] and
+#' @param output List of the output object from [run()] and
 #'   the name of the result file to read. For the error file,
 #'   `.error` is appended to this.
 #' @param options The context, including all parameters.
@@ -20,7 +20,7 @@ get_result <- function(output, options) {
   res <- options$result_file
 
   ## Timeout?
-  if (output$timeout) throw(new_callr_error(output))
+  if (output$timeout) throw(new_callr_crash_error(output))
 
   ## No output file and no error file? Some other (system?) error then,
   ## unless exit status was zero, which is probably just quit().
@@ -32,14 +32,14 @@ get_result <- function(output, options) {
     "has crashed or was killed")
   if (! file.exists(res) && ! file.exists(errorres)) {
     if (is.na(output$status) || output$status != 0) {
-      throw(new_callr_error(output, killmsg))
+      throw(new_callr_crash_error(output, killmsg))
     } else  {
       return(ret)
     }
   }
 
   ## No error file? Then probably all is well, return the output
-  ## If this is currupt, then the R process has crashed
+  ## If this is corrupt, then the R process has crashed
   ## This cannot happen from R 3.5.0, because that version only writes
   ## out the output file if no error or crash has happened.
   ## (Older R versions write a corrupt RDS file in this case.)
@@ -48,7 +48,12 @@ get_result <- function(output, options) {
       ret <- readRDS(res),
       error = function(e) {
         if (is.na(output$status) || output$status != 0) {
-          throw(new_callr_error(output, killmsg))
+          throw(new_callr_crash_error(output, killmsg))
+        } else {
+          throw(new_callr_crash_error(
+            output,
+            "could not read result from callr"
+          ))
         }
       }
     )
@@ -56,29 +61,33 @@ get_result <- function(output, options) {
     return(ret)
   }
 
+  # To work around an errors.R bug, if the format method is registered
+  # from another package. errors.R expects that the parent error has a
+  # message, but if it is an interrupt (and perhaps some other rare cases),
+  # it does not.
+  fix_msg <- function(cnd) {
+    if (is.null(cnd$message)) {
+      if (inherits(cnd, "interrupt")) {
+        cnd$message <- "interrupt"
+      } else {
+        cnd$message <- ""
+      }
+    }
+    cnd
+  }
+
   ## The error RDS might be corrupt, too, if we crashed/got killed after
   ## an error
   tryCatch(
     remerr <- readRDS(errorres),
-    error = function(e) throw(new_callr_error(output, killmsg))
+    error = function(e) throw(new_callr_crash_error(output, killmsg))
   )
 
   if (remerr[[1]] == "error") {
-    remerr[[2]]$message <- remerr[[2]]$message %||% "interrupt"
-    msg <- conditionMessage(remerr[[2]]$error)
-    newerr <- new_callr_error(output, msg)
-    throw(newerr, parent = remerr[[2]])
+    throw(callr_remote_error(remerr, output), parent = fix_msg(remerr[[3]]))
 
   } else if (remerr[[1]] == "stack") {
-    myerr <- structure(
-      list(
-        message = conditionMessage(remerr[[2]]),
-        call = conditionCall(remerr[[2]]),
-        stack = clean_stack(remerr[[3]])
-      ),
-      class = c("callr_error", "error", "condition")
-    )
-    throw(myerr)
+    throw(callr_remote_error_with_stack(remerr, output), parent = fix_msg(remerr[[2]]))
 
   } else if (remerr[[1]] == "debugger") {
     utils::debugger(clean_stack(remerr[[3]]))
